<?php

declare(strict_types=1);

namespace mirzaev\vk\chat;

// Файлы проекта
use mirzaev\vk\chat\core,
    mirzaev\vk\chat\program;

/**
 * Команда
 *
 * @package mirzaev\vk\chat
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class command
{
    /**
     * @var array $programs Программы
     */
    protected array $programs = [];

    /**
     * Конструктор
     *
     * @param string $text Текст
     */
    public function __construct(public string $text)
    {
    }

    /**
     * Записать программу в реестр
     *
     * @param program $program Инстанция программы
     *
     * @return static
     */
    public function program(program $program): static
    {
        // Запись в реестр
        $this->programs[] = $program;

        return $this;
    }

    /**
     * Проверить текст на совпадение с шаблоном
     *
     * @param string $text Текст
     * @param array &$parameters Буфер параметров для программы
     *
     * @return bool Текст совпадает с шаблоном?
     */
    public function check(string $text, array &$parameters = []): bool
    {
        // Проверка на совпадение с шаблоном
        preg_match_all($this->text, $text, $matches, PREG_PATTERN_ORDER);

        // Проверка на нахождение
        if (empty($matches[0])) return false;

        // Генерация параметров
        for ($i = 0, $count = count($matches) + 1; $i <= $count; $i++) foreach ($matches ?? [] as $match) if (!empty($match[$i])) $parameters[] = $match[$i];

        return true;
    }

    /**
     * Обработать подключенные программы
     *
     * @param core $core Ядро модуля
     * @param int $id Идентификатор аккаунта вызывающего выполнение
     * @param array &$update Событие
     * @param string ...$parameters Параметры программы
     *
     * @return void
     */
    public function handle(core $core, int $id, array &$update, string ...$parameters): void
    {
        foreach ($this->programs as $program) {
            // Перебор программ

            // Авторизация
            if (!empty($program->accounts) && !$program->access($id)) return;

            // Выполнение программы
            $program->execute($core, $update, ...$parameters);
        }
    }
}
