<?php

declare(strict_types=1);

namespace mirzaev\vk\chat;

// Фреймворк ВКонтакте
use mirzaev\vk\robots\robot;

// Встроенные библиотеки
use Closure;

/**
 * Программа
 *
 * @package mirzaev\vk\chat
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class program
{
    /**
     * Конструктор
     *
     * @param callable $program Программа выполняемая командой
     * @param robot $robot Робот, который будет выполнять команду
     * @param array $accounts Реестр аккаунтов которым разрешено выполнять команду
     */
    public function __construct(
        public Closure $program,
        public robot $robot,
        public array $accounts = []
    ) {
    }

    /**
     * Проверить наличие доступа к команде
     *
     * @param int $id Идентификатор аккаунта
     *
     * @return bool Имеет ли аккаунт доступ к команде?
     */
    public function access(int $id): bool
    {
        // Перебор реестра аккаунтов
        foreach ($this->accounts as $account) if ($account === $id) return true;

        return false;
    }

    /**
     * Выполнить команду
     *
     * @param core $core Ядро модуля
     * @param array &$update Событие
     * @param string ...$parameters Параметры команды
     *
     * @return mixed
     */
    public function execute(core $core, array &$update, string ...$parameters): mixed
    {
        return call_user_func_array($this->program, [$core, $this->robot, &$update, ...$parameters]);
    }
}
